@@ -6,6 +6,8 @@ import traceback
 from functools import lru_cache
 from typing import Any, List, Optional, Tuple, Union
 
+from ._version import __author__, __version__  # noqa: F401
+
 
 def get_brightness(
     display: Optional[Union[int, str]] = None,
@@ -566,7 +568,7 @@ class Monitor():
 
             # check if the benq monitor is the primary one
             if primary.serial == benq_monitor.serial:
-                print('BenQ GL2450HM is the primary display')
+                print('BenQ GL2450H is the primary display')
             else:
                 print('The primary display is', primary.name)
 
@@ -1044,6 +1046,3 @@ elif plat == 'Darwin':
 else:
     raise NotImplementedError(f'{plat} is not yet supported')
 del plat
-
-__version__ = '0.11.2'
-__author__ = 'Crozzers'
