@@ -8,6 +8,7 @@ from typing import List, Optional, Union
 
 import pythoncom
 import win32api
+import win32con
 import wmi
 
 from . import EDID, __cache__, _monitor_brand_lookup, filter_monitors, platform
@@ -28,6 +29,26 @@ def _wmi_init():
     return wmi.WMI(namespace='wmi')
 
 
+def enum_display_devices() -> Generator[win32api.PyDISPLAY_DEVICEType, None, None]:
+    '''
+    Yields all display devices connected to the computer
+
+    Yields:
+        win32api.PyDISPLAY_DEVICEType
+    '''
+    for monitor_enum in win32api.EnumDisplayMonitors():
+        pyhandle = monitor_enum[0]
+        monitor_info = win32api.GetMonitorInfo(pyhandle)
+        for adaptor_index in range(5):
+            try:
+                device = win32api.EnumDisplayDevices(monitor_info['Device'], adaptor_index, 1)
+            except Exception:
+                pass
+            else:
+                yield device
+                break
+
+
 def get_display_info() -> List[dict]:
     '''
     Gets information about all connected displays using WMI and win32api
@@ -46,33 +67,12 @@ def get_display_info() -> List[dict]:
     '''
     info = __cache__.get('windows_monitors_info_raw')
     if info is None:
+        info = []
         try:
             # collect all monitor UIDs (derived from DeviceID)
             monitor_uids = {}
-            last_good = 0
-            for i in win32api.EnumDisplayMonitors():
-                tmp = win32api.GetMonitorInfo(i[0])
-                try:
-                    # If a user has all displays plugged into display adapter 4 then
-                    # this statement would be tried for each adapter (0-4) per monitor
-                    # meaning 4 tries per monitor, 12 tries total.
-                    # By trying the "last good" adapter first, only the first display will
-                    # be tried 4 times. All the others will be tried once
-                    device = win32api.EnumDisplayDevices(tmp['Device'], last_good, 1)
-                    monitor_uids[device.DeviceID.split('#')[2]] = device
-                except Exception:
-                    # iterate up to 4 display adapters
-                    for j in range(5):
-                        try:
-                            # last_good is tried just above here. No point trying it again
-                            if j != last_good:
-                                # i have no idea what this 3rd parameter does but the code doesn't work without it
-                                device = win32api.EnumDisplayDevices(tmp['Device'], j, 1)
-                                monitor_uids[device.DeviceID.split('#')[2]] = device
-                                last_good = j
-                                break
-                        except Exception:
-                            continue
+            for device in enum_display_devices():
+                monitor_uids[device.DeviceID.split('#')[2]] = device
 
             # gather list of laptop displays to check against later
             wmi = _wmi_init()
@@ -326,7 +326,20 @@ class VCP:
         if not windll.user32.EnumDisplayMonitors(None, None, cls._MONITORENUMPROC(callback), None):
             raise WinError('EnumDisplayMonitors failed')
 
-        index = 0
+        # user index keeps track of valid monitors
+        user_index = 0
+        # monitor index keeps track of valid and pseudo monitors
+        monitor_index = 0
+        display_devices = list(enum_display_devices())
+
+        wmi = _wmi_init()
+        try:
+            laptop_displays = [
+                i.InstanceName.replace('_0', '').split('\\')[2]
+                for i in wmi.WmiMonitorBrightness()
+            ]
+        except Exception:
+            laptop_displays = []
 
         for monitor in monitors:
             # Get physical monitor count
@@ -334,18 +347,24 @@ class VCP:
             if not windll.dxva2.GetNumberOfPhysicalMonitorsFromHMONITOR(monitor, byref(count)):
                 raise WinError()
             if count.value > 0:
-                if start and index + count.value < start:
-                    index += count.value
-                    continue
                 # Get physical monitor handles
                 physical_array = (cls._PHYSICAL_MONITOR * count.value)()
                 if not windll.dxva2.GetPhysicalMonitorsFromHMONITOR(monitor, count.value, physical_array):
                     raise WinError()
                 for item in physical_array:
-                    if not (start and index != start):
-                        yield item.handle
+                    if display_devices[monitor_index].StateFlags & win32con.DISPLAY_DEVICE_ATTACHED_TO_DESKTOP:
+                        # check that the monitor is not a pseudo monitor by
+                        # checking it's StateFlags for the
+                        # win32con DISPLAY_DEVICE_ATTACHED_TO_DESKTOP flag
+                        if display_devices[monitor_index].DeviceID.split('#')[2] not in laptop_displays:
+                            # check if monitor is actually a laptop display
+                            if not (start and user_index != start):
+                                yield item.handle
+                            # increment user index as a valid monitor was found
+                            user_index += 1
+                    # increment monitor index
+                    monitor_index += 1
                     windll.dxva2.DestroyPhysicalMonitor(item.handle)
-                    index += 1
 
     @classmethod
     def get_display_info(cls, display: Optional[Union[int, str]] = None) -> List[dict]:
@@ -382,12 +401,14 @@ class VCP:
         return info
 
     @classmethod
-    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
+    def get_brightness(cls, display: Optional[int] = None, max_tries: int = 50) -> List[int]:
         '''
         Retrieve the brightness of all connected displays using the `ctypes.windll` API
 
         Args:
             display (int): The specific display you wish to query.
+            max_tries (int): the maximum allowed number of attempts to
+                read the VCP output from the monitor
 
         Returns:
             list: list of ints (0 to 100)
@@ -417,12 +438,12 @@ class VCP:
             if current is None:
                 cur_out = DWORD()
                 handle = HANDLE(monitor)
-                for _ in range(10):
+                for attempt in range(max_tries):
                     if windll.dxva2.GetVCPFeatureAndVCPFeatureReply(handle, code, None, byref(cur_out), None):
                         current = cur_out.value
                         break
                     current = None
-                    time.sleep(0.02)
+                    time.sleep(0.02 if attempt < 20 else 0.1)
 
             if current is not None:
                 __cache__.store(f'vcp_brightness_{index}', current, expires=0.1)
@@ -436,12 +457,14 @@ class VCP:
         return values
 
     @classmethod
-    def set_brightness(cls, value: int, display: Optional[int] = None):
+    def set_brightness(cls, value: int, display: Optional[int] = None, max_tries: int = 50):
         '''
         Sets the brightness for all connected displays using the `ctypes.windll` API
 
         Args:
             display (int): The specific display you wish to query.
+            max_tries (int): the maximum allowed number of attempts to
+                send the VCP input to the monitor
 
         Examples:
             ```python
@@ -466,10 +489,10 @@ class VCP:
         ):
             if display is None or display == index:
                 handle = HANDLE(monitor)
-                for _ in range(10):
+                for attempt in range(max_tries):
                     if windll.dxva2.SetVCPFeature(handle, code, value):
                         break
-                    time.sleep(0.02)
+                    time.sleep(0.02 if attempt < 20 else 0.1)
 
 
 def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = False) -> List[dict]:
